# -*- coding: utf-8 -*-
#MoxCycleKey for Blender 2023
import bpy

bl_info = {
    "name": "MoxCycleKey",
    "author": "Mox co,.ltd",
    "version": (1, 0),
    "blender": (2, 83, 0),
    "location": "3Dビューポート > オブジェクト > アニメーション",
    "description": "F-Curveのコピペでサイクルアニメーションを実現します。",
    "warning": "",
    "support": "COMMUNITY",
    "doc_url": "",
    "tracker_url": "",
    "category": "Animation"
}

# メインオペレータ
class Mox_OT_CycleKey(bpy.types.Operator):
    bl_idname = "mox.cyclekey" 
    bl_label = "MoxCycleKey"
    bl_description = "F-Curveのコピペでサイクルアニメーションを実現します。"
    bl_options = {'REGISTER', 'UNDO'}

    # メニューを実行したときに呼ばれるメソッド
    def execute(self, context):
        # ================= ポーズモードなら ======================
        if bpy.context.mode == "POSE":
            armature = bpy.context.object
            if armature:
                fselcount = 0
                pose_bones = bpy.context.selected_pose_bones
                if pose_bones:
                    for pb in pose_bones:#選択したすべてのボーンから一つずつ
                        fcvs =  armature.animation_data.action.fcurves
                        if fcvs is not None:
                            for fcv in fcvs:#取得したFカーブリストから、一本だけＦカーブを取り出し個別に処理
                                if pb.name in fcv.data_path and fcv.select:
                                    fselcount +=1
                                    lastKey = fcv.keyframe_points[-1].co.x  #最後のキーフレームを取得
                                    diffKey = lastKey - fcv.keyframe_points[0].co.x #最後のキーフレームから最初のキーフレームを引くと差分が取得できる
                                    for key in fcv.keyframe_points:
                                        fcv.keyframe_points.insert(diffKey + key.co.x,  key.co.y)
                            if fselcount == 0:
                                self.report({'ERROR'}, u"F-Curveを選択してください。")
                        else:
                            self.report({'ERROR'}, u"{}にはF-Curveが存在しません。".format(pb.name))
                        bpy.ops.wm.redraw_timer(type='DRAW_WIN_SWAP', iterations=1) #再描画
                else:
                    self.report({'ERROR'}, u"ボーンまたはボーンコントローラーを選択してください")
            else:
                self.report({'ERROR'}, u"armatureが存在しません")

            
        # ================= オブジェクトモードなら ======================
        else:
            sels = bpy.context.selected_objects #すべての選択オブジェクト
            fselcount = 0
            if len(sels) != 0:
                for sel in sels:
                    if sel.animation_data and sel.animation_data.action:
                        fcvs = sel.animation_data.action.fcurves
                        if fcvs is not None:
                            for fcv in fcvs:#取得したFカーブリストから、一本だけＦカーブを取り出し個別に処理
                                # 選択したFカーブに対して処理
                                if fcv.select:
                                    fselcount +=1
                                    lastKey = fcv.keyframe_points[-1].co.x  #最後のキーフレームを取得
                                    diffKey = lastKey - fcv.keyframe_points[0].co.x #最後のキーフレームから最初のキーフレームを引くと差分が取得できる
                                    for key in fcv.keyframe_points:
                                        fcv.keyframe_points.insert(diffKey + key.co.x,  key.co.y)
                                
                            if fselcount == 0:
                                self.report({'ERROR'}, u"F-Curveを選択してください。")
                        else:
                            self.report({'ERROR'}, u"{}にはF-Curveが存在しません。".format(sel.name))
                    else:
                        self.report({'ERROR'}, u"アニメーションデータが存在しません")
                    bpy.ops.wm.redraw_timer(type='DRAW_WIN_SWAP', iterations=1) #再描画
            else:
                self.report({'ERROR'}, u"オブジェクトを選択してください")

        return {'FINISHED'}

def menu_fn(self, context):
    self.layout.operator(Mox_OT_CycleKey.bl_idname)


classes = [
    Mox_OT_CycleKey
]

def register():
    for c in classes:
        bpy.utils.register_class(c)
    bpy.types.VIEW3D_MT_object_animation.append(menu_fn)
    bpy.types.VIEW3D_MT_object_animation.path_menu


def unregister():
    bpy.types.VIEW3D_MT_object_animation.remove(menu_fn)
    for c in classes:
        bpy.utils.unregister_class(c)

if __name__ == "__main__":
    register()